(ns wanijo.instance.routes
  (:require [compojure.core :refer [defroutes GET POST DELETE]]
            [ring.util.response :as resp]
            [formulare.core :as form]
            [wanijo.instance.view :as view]
            [wanijo.instance.domain :as domain]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.schema.domain :as domain-schema]
            [wanijo.attribute.domain :as domain-attr]))

(defn attr-type->widget [attr-type]
  (case attr-type
    ("markdown" "text") :textarea
    :input))

(defn attr->field-id [attr]
  (keyword (str "attr-" (:name attr))))

(defn attr->field [attr]
  {:label (:name attr)
   :required true
   :widget (attr-type->widget (:type attr))})

(defn new-form [schema-uuid]
  (update view/new-form
          :fields
          (fn [fields]
            (reduce (fn [fields attr]
                      (assoc fields
                             (attr->field-id attr)
                             (attr->field attr)))
                    fields
                    (domain-attr/required! schema-uuid)))))

(defn list! [schema-uuid req]
  (view/list! (domain-schema/find-by-uuid! schema-uuid)
              (domain/find-by-schema! schema-uuid)
              (new-form schema-uuid)
              req))

(defn form-data->instance [form-data required-attrs]
  {:name (:name form-data)
   :properties (map (fn [ra]
                      {:attribute ra
                       :value ((attr->field-id ra) form-data)})
                    required-attrs)})

(defn new! [req]
  (let [schema-uuid (get-in req [:params :schema-uuid])
        form-def (new-form schema-uuid)]
    (if (form/valid? form-def req)
      (let [instance (form-data->instance
                      (form/form-data form-def req)
                      (domain-attr/required! schema-uuid))]
        (domain/create! schema-uuid
                        instance)
        (resp/redirect (path :instance-list
                             (:params req))))
      (list! schema-uuid req))))

(defroutes routes
  (GET (register! :instance-list "/instance/list/:schema-uuid")
       [schema-uuid :as req]
       (list! schema-uuid req))
  (POST (register! :instance-new "/instance/new") []
        new!))
